//`define EXCHANGE_12_34

module DsiTest_Top
(   input           RESET_N
,   input           OSC_50M
,   inout           ck_n
,   inout           ck_p
,   inout           d0_n
,   inout           d0_p
,   inout           d1_n
,   inout           d1_p
,   inout           d2_n
,   inout           d2_p
,   inout           d3_n
,   inout           d3_p
,   output [2:0]    led
);
localparam  WC_REF  = 16'd5760  ; //! 1920*3 = 5760;
localparam  DT_RGB  = 6'h3E     ;

//======================================================
reg  [31:0] run_cnt     ;
reg         pixel_vs_r  ;
reg  [10:0] vs_cnt      ;
reg         running     ;
reg  [4:0]  frame       ;
reg         frame_led   ;
//
//wire        bit_clk     ;
//wire        bit_clk_90  ;
wire        byte_clk    ;
wire        pixel_clk   ;

wire        pixel_de    /* synthesis syn_keep = 1 */;
wire        pixel_hs    /* synthesis syn_keep = 1 */;
wire        pixel_vs    /* synthesis syn_keep = 1 */;
wire        vs_pulse    /* synthesis syn_keep = 1 */;
wire [7:0]  pixel_r     /* synthesis syn_keep = 1 */;
wire [7:0]  pixel_g     /* synthesis syn_keep = 1 */;
wire [7:0]  pixel_b     /* synthesis syn_keep = 1 */;
wire [23:0] pixel       = {pixel_b, pixel_g, pixel_r};
//
wire        vs_start    /* synthesis syn_keep = 1 */;
wire        vs_end      /* synthesis syn_keep = 1 */;
wire        hs_start    /* synthesis syn_keep = 1 */;
wire        hs_end      /* synthesis syn_keep = 1 */;
wire        data_en     /* synthesis syn_keep = 1 */;
wire [31:0] data        /* synthesis syn_keep = 1 */;
//
wire        hs_clk_en   /* synthesis syn_keep = 1 */;
wire [7:0]  hs_clk      /* synthesis syn_keep = 1 */;
wire [1:0]  lp_clk      /* synthesis syn_keep = 1 */;
wire        hs_data_en  /* synthesis syn_keep = 1 */;
wire [7:0]  hs_data0    /* synthesis syn_keep = 1 */;
wire [7:0]  hs_data1    /* synthesis syn_keep = 1 */;
wire [7:0]  hs_data2    /* synthesis syn_keep = 1 */;
wire [7:0]  hs_data3    /* synthesis syn_keep = 1 */;
wire [1:0]  lp_data0    ;
//
reg         hs_clk_en_r ;
reg  [7:0]  hs_clk_r    ;
reg  [1:0]  lp_clk_r    ;
reg         hs_data_en_r;
reg  [7:0]  hs_data0_r  ;
reg  [7:0]  hs_data1_r  ;
reg  [7:0]  hs_data2_r  ;
reg  [7:0]  hs_data3_r  ;
reg  [1:0]  lp_data0_r  ;

//=============================================================================
// running logic
always @(posedge pixel_clk or negedge RESET_N)
if (!RESET_N)   run_cnt  <= 'b0;
else            run_cnt  <= run_cnt + 1'd1;

always @(posedge pixel_clk)
running <= run_cnt[25];

//=============================================================================
// testpattern
testpattern u_test_gen
(   .I_pxl_clk      (pixel_clk     )   //! pixel clock 
,   .I_rst_n        (RESET_N       )   //! low active 
,   .I_mode         (vs_cnt[10:8]  )   //! data pattern select
,   .I_single_r     (8'd0          )
,   .I_single_g     (8'd0          )
,   .I_single_b     (8'd255        )
                                       //! 141.667 MHZ         1920x1080
,   .I_h_total      (12'd2163      )   //! hor total time : 12'd2163
,   .I_h_sync       (12'd64        )   //! hor sync time  : 12'd64  
,   .I_h_bporch     (12'd64        )   //! hor back porch : 12'd64  
,   .I_h_res        (12'd1920      )   //! hor resolution : 12'd1920
,   .I_v_total      (12'd1092      )   //! ver total time : 12'd1092
,   .I_v_sync       (12'd4         )   //! ver sync time  : 12'd4   
,   .I_v_bporch     (12'd4         )   //! ver back porch : 12'd4   
,   .I_v_res        (12'd1080      )   //! ver resolution : 12'd1080
,   .I_hs_pol       (1'b1          )
,   .I_vs_pol       (1'b1          )
,   .O_de           (pixel_de      )   
,   .O_hs           (pixel_hs      )
,   .O_vs           (pixel_vs      )
,   .O_data_r       (pixel_r       )   
,   .O_data_g       (pixel_g       )
,   .O_data_b       (pixel_b       )
);

//=============================================================================
// pattern changing logic
always @(posedge pixel_clk or negedge RESET_N) begin
if (~RESET_N)       pixel_vs_r  <= 1'b0;
else                pixel_vs_r  <= pixel_vs;

if (~RESET_N)       vs_cnt      <= 'b0;
else if (vs_pulse)  vs_cnt[9:0] <= vs_cnt[9:0] + 1'd1;

if (~RESET_N)       frame   <= 'b0;
else if (vs_pulse)  frame   <= (frame == 5'd29) ? 5'd0 : frame + 1'd1;

if (~RESET_N)       frame_led   <= 1'b0;
else if (vs_pulse)  frame_led   <= frame==5'd29 ? ~frame_led : frame_led;

end

assign vs_pulse    = pixel_vs & ~pixel_vs_r;

//=============================================================================
// Piel2Byte
Pixel_to_Byte_Top u_p2b
(   .I_RSTN         (RESET_N        )
,   .I_PIXEL_CLK    (pixel_clk      )
,   .I_BYTE_CLK     (byte_clk       )
,   .I_VSYNC        (pixel_vs       )
,   .I_HSYNC        (pixel_hs       )
,   .I_DE           (pixel_de       )
,   .I_PIXEL        (pixel          )
,   .O_VS_START     (vs_start       )
,   .O_VS_END       (vs_end         )
,   .O_HS_START     (hs_start       )
,   .O_HS_END       (hs_end         )
,   .O_DATA_EN      (data_en        )
,   .O_DATA         (data           )
);

//=============================================================================
// DSI TX
MIPI_DSI_TX_Top u_dsi_tx
(   .I_RSTN         (RESET_N        )
,   .I_BYTE_CLK     (byte_clk       )

,   .I_VS_START     (vs_start       )
,   .I_VS_END       (vs_end         )
,   .I_HS_START     (hs_start       )
,   .I_HS_END       (hs_end         )

,   .I_DATA_EN      (data_en        )
,   .I_DATA         (data           )
,   .I_WC           (data_en ? WC_REF : 16'd0 )
,   .I_VC           (2'b00          )
,   .I_DT           (DT_RGB         )

,   .O_HS_CLK_EN    (hs_clk_en      )
,   .O_HS_CLK       (hs_clk         )
,   .O_LP_CLK       (lp_clk         )
,   .O_HS_DATA_EN   (hs_data_en     )
,   .O_HS_DATA0     (hs_data0       )
,   .O_HS_DATA1     (hs_data1       )
,   .O_HS_DATA2     (hs_data2       )
,   .O_HS_DATA3     (hs_data3       )
,   .O_LP_DATA0     (lp_data0       )
);

`ifdef DEBUG
Gowin_PLL u_pll
(   .reset          (~RESET_N       ) //input reset
,   .clkin          ( OSC_50M       ) //input clkin
,   .lock           (               ) //output lock
,   .clkout0        ( byte_clk      ) //output clkout0
,   .clkout1        ( pixel_clk     ) //output clkout1
);
`else

reg  tx_drst_n   = 1'b0;
always @(posedge byte_clk or negedge RESET_N) begin
    if (~RESET_N)   tx_drst_n   <= 1'b0;
    else            tx_drst_n   <= ~(lp_data0 == 2'b01);
end


Gowin_MIPI_DPHY_TX_Top u_dphy_tx_ip
(   .clkin          ( OSC_50M       )   //input clkin
,   .clkout4        ( pixel_clk     )   //output clkout4, VCO/6 = 850/6 = 141.67M, for pixel clock

,   .tx_clk_o       ( byte_clk      )   //output tx_clk_o, Byte clock
,   .tx_drst_n      ( tx_drst_n     )   //input tx_drst_n

,   .ck_n           ( ck_n          )   //inout ck_n
,   .ck_p           ( ck_p          )   //inout ck_p
,   .d0_n           ( d0_n          )   //inout d0_n
,   .d0_p           ( d0_p          )   //inout d0_p
,   .d1_n           ( d1_n          )   //inout d1_n
,   .d1_p           ( d1_p          )   //inout d1_p
,   .d2_n           ( d2_n          )   //inout d2_n
,   .d2_p           ( d2_p          )   //inout d2_p
,   .d3_n           ( d3_n          )   //inout d3_n
,   .d3_p           ( d3_p          )   //inout d3_p

,   .di_lprx0_n     ( ) //output di_lprx0_n
,   .di_lprx0_p     ( ) //output di_lprx0_p
,   .di_lprx1_n     ( ) //output di_lprx1_n
,   .di_lprx1_p     ( ) //output di_lprx1_p
,   .di_lprx2_n     ( ) //output di_lprx2_n
,   .di_lprx2_p     ( ) //output di_lprx2_p
,   .di_lprx3_n     ( ) //output di_lprx3_n
,   .di_lprx3_p     ( ) //output di_lprx3_p
,   .di_lprxck_n    ( ) //output di_lprxck_n
,   .di_lprxck_p    ( ) //output di_lprxck_p
,   .do_lptx0_n     ( lp_data0[0]   )   //input do_lptx0_n
,   .do_lptx1_n     ( lp_data0[0]   )   //input do_lptx1_n
,   .do_lptx2_n     ( lp_data0[0]   )   //input do_lptx2_n
,   .do_lptx3_n     ( lp_data0[0]   )   //input do_lptx3_n
,   .do_lptx0_p     ( lp_data0[1]   )   //input do_lptx0_p
,   .do_lptx1_p     ( lp_data0[1]   )   //input do_lptx1_p
,   .do_lptx2_p     ( lp_data0[1]   )   //input do_lptx2_p
,   .do_lptx3_p     ( lp_data0[1]   )   //input do_lptx3_p
,   .do_lptxck_n    ( lp_clk[0]     )   //input do_lptxck_n
,   .do_lptxck_p    ( lp_clk[1]     )   //input do_lptxck_p
,   .lprx_en_ck     ( 1'b0          )   //input lprx_en_ck
,   .lprx_en_d0     ( 1'b0          )   //input lprx_en_d0
,   .lprx_en_d1     ( 1'b0          )   //input lprx_en_d1
,   .lprx_en_d2     ( 1'b0          )   //input lprx_en_d2
,   .lprx_en_d3     ( 1'b0          )   //input lprx_en_d3

,   .hstxen_lnck    ( hs_clk_en     )    //input hstxen_lnck
,   .lptxen_lnck    (~hs_clk_en     )    //input lptxen_lnck
,   .hstxd_vld      ( 1'b1          )    //input hstxd_vld
,   .hstxen_ln0     ( hs_data_en    )    //input hstxen_ln0
,   .hstxen_ln1     ( hs_data_en    )    //input hstxen_ln1
,   .hstxen_ln2     ( hs_data_en    )    //input hstxen_ln2
,   .hstxen_ln3     ( hs_data_en    )    //input hstxen_ln3
,   .lptxen_ln0     (~hs_data_en    )    //input lptxen_ln0
,   .lptxen_ln1     (~hs_data_en    )    //input lptxen_ln1
,   .lptxen_ln2     (~hs_data_en    )    //input lptxen_ln2
,   .lptxen_ln3     (~hs_data_en    )    //input lptxen_ln3
,   .txdpen_ln0     ( 1'b1          )    //input txdpen_ln0
,   .txdpen_ln1     ( 1'b1          )    //input txdpen_ln1
,   .txdpen_ln2     ( 1'b1          )    //input txdpen_ln2
,   .txdpen_ln3     ( 1'b1          )    //input txdpen_ln3
,   .txdpen_lnck    ( 1'b1          )    //input txdpen_lnck
,   .txhclk_en      ( 1'b1          )    //input txhclk_en
,   .ckln_hstxd     ( hs_clk        )   //input [7:0] ckln_hstxd
`ifdef EXCHANGE_12_34
,   .d0ln_hstxd     ( hs_data2      )   //input [7:0] d0ln_hstxd
,   .d1ln_hstxd     ( hs_data3      )   //input [7:0] d1ln_hstxd
,   .d2ln_hstxd     ( hs_data0      )   //input [7:0] d2ln_hstxd
,   .d3ln_hstxd     ( hs_data1      )   //input [7:0] d3ln_hstxd
`else
,   .d0ln_hstxd     ( hs_data0      )   //input [7:0] d0ln_hstxd
,   .d1ln_hstxd     ( hs_data1      )   //input [7:0] d1ln_hstxd
,   .d2ln_hstxd     ( hs_data2      )   //input [7:0] d2ln_hstxd
,   .d3ln_hstxd     ( hs_data3      )   //input [7:0] d3ln_hstxd
`endif
);
`endif 

assign  led[0]  = running   ;
assign  led[1]  = frame_led ;

//! for DEBUG
always @ (posedge byte_clk) begin
    hs_clk_en_r     <= hs_clk_en   ;
    hs_clk_r        <= hs_clk      ;
    lp_clk_r        <= lp_clk      ;
    hs_data_en_r    <= hs_data_en  ;
    hs_data0_r      <= hs_data0    ;
    hs_data1_r      <= hs_data1    ;
    hs_data2_r      <= hs_data2    ;
    hs_data3_r      <= hs_data3    ;
    lp_data0_r      <= lp_data0    ;
end

assign  led[2]  = 1'b0
                | hs_clk_en_r
                | ^hs_clk_r     
                | ^lp_clk_r     
                | hs_data_en_r 
                | ^hs_data0_r   
                | ^hs_data1_r   
                | ^hs_data2_r   
                | ^hs_data3_r   
                | ^lp_data0_r
                ;
endmodule