
module DsiTest_Top
(   input           RESET_N
,   input           OSC_50M

,   inout           TX_CLK_P
,   inout           TX_CLK_N
,   inout           TX_D0_P
,   inout           TX_D0_N
,   inout           TX_D1_P
,   inout           TX_D1_N
,   inout           TX_D2_P
,   inout           TX_D2_N
,   inout           TX_D3_P
,   inout           TX_D3_N

,   output [2:0]    led
);
localparam  WC_REF  = 16'd5760  ; //! 1920*3 = 5760;
localparam  DT_RGB  = 6'h3E     ;

//======================================================
reg  [31:0] run_cnt     ;
reg         pixel_vs_r  ;
reg  [10:0] vs_cnt      ;
reg         running     ;
reg  [4:0]  frame       ;
reg         frame_led   ;
//
wire        bit_clk     ;
wire        bit_clk_90  ;
wire        byte_clk    ;
wire        pixel_clk   ;

wire        pixel_de    /* synthesis syn_keep = 1 */;
wire        pixel_hs    /* synthesis syn_keep = 1 */;
wire        pixel_vs    /* synthesis syn_keep = 1 */;
wire        vs_pulse    /* synthesis syn_keep = 1 */;
wire [ 7:0] pixel_r     /* synthesis syn_keep = 1 */;
wire [ 7:0] pixel_g     /* synthesis syn_keep = 1 */;
wire [ 7:0] pixel_b     /* synthesis syn_keep = 1 */;
wire [23:0] pixel       = {pixel_b, pixel_g, pixel_r};
//
wire        vs_start    /* synthesis syn_keep = 1 */;
wire        vs_end      /* synthesis syn_keep = 1 */;
wire        hs_start    /* synthesis syn_keep = 1 */;
wire        hs_end      /* synthesis syn_keep = 1 */;
wire        data_en     /* synthesis syn_keep = 1 */;
wire [31:0] data        /* synthesis syn_keep = 1 */;
//
wire        hs_clk_en   /* synthesis syn_keep = 1 */;
wire [ 7:0] hs_clk      /* synthesis syn_keep = 1 */;
wire [1:0]  lp_clk      /* synthesis syn_keep = 1 */;
wire        hs_data_en  /* synthesis syn_keep = 1 */;
wire [ 7:0] hs_data0    /* synthesis syn_keep = 1 */;
wire [ 7:0] hs_data1    /* synthesis syn_keep = 1 */;
wire [ 7:0] hs_data2    /* synthesis syn_keep = 1 */;
wire [ 7:0] hs_data3    /* synthesis syn_keep = 1 */;
wire [ 1:0] lp_data0    ;
//
reg         hs_clk_en_r ;
reg  [ 7:0] hs_clk_r    ;
reg  [1:0]  lp_clk_r    ;
reg         hs_data_en_r;
reg  [ 7:0] hs_data0_r  ;
reg  [ 7:0] hs_data1_r  ;
reg  [ 7:0] hs_data2_r  ;
reg  [ 7:0] hs_data3_r  ;
reg  [ 1:0] lp_data0_r  ;

//=============================================================================
// PLL
Gowin_PLL u_pll
(   .reset      (~RESET_N   )   //! input reset
,   .clkin      (OSC_50M    )   //! in 50 MHz
,   .clkout0    (bit_clk    )   //! x17/2 = 425 MHz
,   .clkout1    (bit_clk_90 )   //! 90
,   .clkout2    (byte_clk   )   //! / 4 = 106.25 MHz
,   .clkout3    (pixel_clk  )   //! / 3 = 141.67 MHz
,   .lock       (           )   //! output lock
);

//=============================================================================
// running logic
always @(posedge pixel_clk or negedge RESET_N)
if (!RESET_N)   run_cnt  <= 'b0;
else            run_cnt  <= run_cnt + 1'd1;

always @(posedge pixel_clk)
running <= run_cnt[25];

//=============================================================================
// testpattern
testpattern u_test_gen
(   .I_pxl_clk      (pixel_clk     )   //! pixel clock 
,   .I_rst_n        (RESET_N       )   //! low active 
,   .I_mode         (vs_cnt[10:8]  )   //! data pattern select
,   .I_single_r     (8'd0          )
,   .I_single_g     (8'd0          )
,   .I_single_b     (8'd255        )
                                       //! 141.67 MHZ       1920x1080
,   .I_h_total      (12'd2163      )   //! hor total time : 12'd2163
,   .I_h_sync       (12'd64        )   //! hor sync time  : 12'd64
,   .I_h_bporch     (12'd64        )   //! hor back porch : 12'd64
,   .I_h_res        (12'd1920      )   //! hor resolution : 12'd1920
,   .I_v_total      (12'd1092      )   //! ver total time : 12'd1092 
,   .I_v_sync       (12'd4         )   //! ver sync time  : 12'd4   
,   .I_v_bporch     (12'd4         )   //! ver back porch : 12'd4   
,   .I_v_res        (12'd1080      )   //! ver resolution : 12'd1080
,   .I_hs_pol       (1'b1          )
,   .I_vs_pol       (1'b1          )
,   .O_de           (pixel_de      )   
,   .O_hs           (pixel_hs      )
,   .O_vs           (pixel_vs      )
,   .O_data_r       (pixel_r       )   
,   .O_data_g       (pixel_g       )
,   .O_data_b       (pixel_b       )
);

//=============================================================================
// pattern changing logic
always @(posedge pixel_clk or negedge RESET_N) begin
if (~RESET_N)       pixel_vs_r  <= 1'b0;
else                pixel_vs_r  <= pixel_vs;

if (~RESET_N)       vs_cnt      <= 'b0;
else if (vs_pulse)  vs_cnt[9:0] <= vs_cnt[9:0] + 1'd1;

if (~RESET_N)       frame   <= 'b0;
else if (vs_pulse)  frame   <= (frame == 5'd29) ? 5'd0 : frame + 1'd1;

if (~RESET_N)       frame_led   <= 1'b0;
else if (vs_pulse)  frame_led   <= frame==5'd29 ? ~frame_led : frame_led;

end

assign vs_pulse    = pixel_vs & ~pixel_vs_r;


//=============================================================================
// Piel2Byte
Pixel_to_Byte_Top u_p2b
(   .I_RSTN         (RESET_N        )
,   .I_PIXEL_CLK    (pixel_clk      )
,   .I_BYTE_CLK     (byte_clk       )
,   .I_VSYNC        (pixel_vs       )
,   .I_HSYNC        (pixel_hs       )
,   .I_DE           (pixel_de       )
,   .I_PIXEL        (pixel          )
,   .O_VS_START     (vs_start       )
,   .O_VS_END       (vs_end         )
,   .O_HS_START     (hs_start       )
,   .O_HS_END       (hs_end         )
,   .O_DATA_EN      (data_en        )
,   .O_DATA         (data           )
);


//=============================================================================
// DSI TX
DSI_TX_Top u_dsi_tx
(   .I_RSTN         (RESET_N        )
,   .I_BYTE_CLK     (byte_clk       )

,   .I_VS_START     (vs_start       )
,   .I_VS_END       (vs_end         )
,   .I_HS_START     (hs_start       )
,   .I_HS_END       (hs_end         )

,   .I_DATA_EN      (data_en        )
,   .I_DATA         (data           )
,   .I_WC           (data_en?WC_REF:16'd0)
,   .I_VC           (2'b00          )
,   .I_DT           (DT_RGB         )

,   .O_HS_CLK_EN    (hs_clk_en      )
,   .O_HS_CLK       (hs_clk         )
,   .O_LP_CLK       (lp_clk         )
,   .O_HS_DATA_EN   (hs_data_en     )
,   .O_HS_DATA0     (hs_data0       )
,   .O_HS_DATA1     (hs_data1       )
,   .O_HS_DATA2     (hs_data2       )
,   .O_HS_DATA3     (hs_data3       )
,   .O_LP_DATA0     (lp_data0       )
);

MIPI_TX_Top u_tx_phy
(   .reset_n        (RESET_N        )   //input reset_n
,   .clk_bit        (bit_clk        )   //input clk_bit
,   .clk_bit_90     (bit_clk_90     )   //input clk_bit_90
,   .sclk           (byte_clk       )   //input sclk

,   .hs_clk_en      (hs_clk_en      )   //input hs_clk_en
,   .clk_data       (hs_clk         )   //input [7:0] clk_data
,   .lp_clk_out     (2'b00          )   //input [1:0] lp_clk_out
,   .lp_clk_in      (               )   //output [1:0] lp_clk_in
,   .lp_clk_dir     (1'b1           )   //input lp_clk_dir
,   .hs_data_en     (hs_data_en     )   //input hs_data_en

,   .data_in3       (hs_data3       )   //input [7:0] data_in3
,   .data_in2       (hs_data2       )   //input [7:0] data_in2
,   .data_in1       (hs_data1       )   //input [7:0] data_in1
,   .data_in0       (hs_data0       )   //input [7:0] data_in0

,   .lp_data3_out   (lp_data0       )   //input [1:0] lp_data3_out
,   .lp_data3_dir   (1'b1           )   //input lp_data3_dir
,   .lp_data3_in    (               )   //output [1:0] lp_data3_in
,   .lp_data2_out   (lp_data0       )   //input [1:0] lp_data2_out
,   .lp_data2_dir   (1'b1           )   //input lp_data2_dir
,   .lp_data2_in    (               )   //output [1:0] lp_data2_in
,   .lp_data1_out   (lp_data0       )   //input [1:0] lp_data1_out
,   .lp_data1_dir   (1'b1           )   //input lp_data1_dir
,   .lp_data1_in    (               )   //output [1:0] lp_data1_in
,   .lp_data0_out   (lp_data0       )   //input [1:0] lp_data0_out
,   .lp_data0_dir   (1'b1           )   //input lp_data0_dir
,   .lp_data0_in    (               )   //output [1:0] lp_data0_in

,   .MIPI_CLK_P     (TX_CLK_P       )   //inout MIPI_CLK_P
,   .MIPI_CLK_N     (TX_CLK_N       )   //inout MIPI_CLK_N
,   .MIPI_LANE3_P   (TX_D3_P        )   //inout MIPI_LANE3_P
,   .MIPI_LANE3_N   (TX_D3_N        )   //inout MIPI_LANE3_N
,   .MIPI_LANE2_P   (TX_D2_P        )   //inout MIPI_LANE2_P
,   .MIPI_LANE2_N   (TX_D2_N        )   //inout MIPI_LANE2_N
,   .MIPI_LANE1_P   (TX_D1_P        )   //inout MIPI_LANE1_P
,   .MIPI_LANE1_N   (TX_D1_N        )   //inout MIPI_LANE1_N
,   .MIPI_LANE0_P   (TX_D0_P        )   //inout MIPI_LANE0_P
,   .MIPI_LANE0_N   (TX_D0_N        )   //inout MIPI_LANE0_N
);


always @ (posedge byte_clk) begin
    hs_clk_en_r     <= hs_clk_en   ;
    hs_clk_r        <= hs_clk      ;
    lp_clk_r        <= lp_clk      ;
    hs_data_en_r    <= hs_data_en  ;
    hs_data0_r      <= hs_data0    ;
    hs_data1_r      <= hs_data1    ;
    hs_data2_r      <= hs_data2    ;
    hs_data3_r      <= hs_data3    ;
    lp_data0_r      <= lp_data0    ;
end

assign  led[0]  = running   ;
assign  led[1]  = frame_led ;
assign  led[2]  = 1'b0
                | hs_clk_en_r
                | ^hs_clk_r     
                | ^lp_clk_r     
                | hs_data_en_r 
                | ^hs_data0_r   
                | ^hs_data1_r   
                | ^hs_data2_r   
                | ^hs_data3_r   
                | ^lp_data0_r
                ;
endmodule